@@ -1,8 +1,7 @@
 import asyncio
 import re
 
-import problemdetails
-
+from imbi import errors
 from imbi.endpoints import base
 
 
@@ -11,7 +10,9 @@ class _RequestHandlerMixin:
     ITEM_NAME = 'project'
     ID_KEY = ['id']
     FIELDS = ['id', 'namespace_id', 'project_type_id', 'name', 'slug',
-              'description', 'environments', 'archived']
+              'description', 'environments', 'archived', 'gitlab_project_id',
+              'sentry_project_slug', 'sonarqube_project_key',
+              'pagerduty_service_id']
     TTL = 300
 
     GET_SQL = re.sub(r'\s+', ' ', """\
@@ -28,7 +29,11 @@ class _RequestHandlerMixin:
                a.slug,
                a.description,
                a.environments,
-               a.archived
+               a.archived,
+               a.gitlab_project_id,
+               a.sentry_project_slug,
+               a.sonarqube_project_key,
+               a.pagerduty_service_id
           FROM v1.projects AS a
           JOIN v1.namespaces AS b ON b.id = a.namespace_id
           JOIN v1.project_types AS c ON c.id = a.project_type_id
@@ -57,6 +62,10 @@ class CollectionRequestHandler(_RequestHandlerMixin,
                a.description,
                a.environments,
                a.archived,
+               a.gitlab_project_id,
+               a.sentry_project_slug,
+               a.sonarqube_project_key,
+               a.pagerduty_service_id,
                v1.project_score(a.id) AS project_score
           FROM v1.projects AS a
           JOIN v1.namespaces AS b ON b.id = a.namespace_id
@@ -71,9 +80,11 @@ class CollectionRequestHandler(_RequestHandlerMixin,
           {{WHERE}}""")
 
     FILTER_CHUNKS = {
-        'name': 'to_tsvector(lower(a.name)) @@ %(name)s::tsquery',
+        'name': 'to_tsvector(lower(a.name)) @@ websearch_to_tsquery(%(name)s)',
         'namespace_id': 'b.id = %(namespace_id)s',
-        'project_type_id': 'c.id = %(project_type_id)s'
+        'project_type_id': 'c.id = %(project_type_id)s',
+        'sonarqube_project_key':
+            'a.sonarqube_project_key = %(sonarqube_project_key)s',
     }
 
     SORT_MAP = {
@@ -101,7 +112,7 @@ class CollectionRequestHandler(_RequestHandlerMixin,
         where_chunks = []
         if self.get_query_argument('include_archived', 'false') == 'false':
             where_chunks.append('a.archived IS FALSE')
-        for kwarg in ['namespace_id', 'project_type_id', 'name']:
+        for kwarg in self.FILTER_CHUNKS.keys():
             value = self.get_query_argument(kwarg, None)
             if value is not None:
                 kwargs[kwarg] = value
@@ -156,6 +167,10 @@ class RecordRequestHandler(_RequestHandlerMixin, base.CRUDRequestHandler):
                a.description,
                a.environments,
                a.archived,
+               a.gitlab_project_id,
+               a.sentry_project_slug,
+               a.sonarqube_project_key,
+               a.pagerduty_service_id,
                v1.project_score(a.id)
           FROM v1.projects AS a
           JOIN v1.namespaces AS b ON b.id = a.namespace_id
@@ -229,7 +244,11 @@ class RecordRequestHandler(_RequestHandlerMixin, base.CRUDRequestHandler):
                slug=%(slug)s,
                description=%(description)s,
                environments=%(environments)s,
-               archived=%(archived)s
+               archived=%(archived)s,
+               gitlab_project_id=%(gitlab_project_id)s,
+               sentry_project_slug=%(sentry_project_slug)s,
+               sonarqube_project_key=%(sonarqube_project_key)s,
+               pagerduty_service_id=%(pagerduty_service_id)s
          WHERE id=%(id)s""")
 
     async def get(self, *args, **kwargs):
@@ -246,8 +265,7 @@ class RecordRequestHandler(_RequestHandlerMixin, base.CRUDRequestHandler):
                     self.GET_URLS_SQL, query_args, 'get-project-urls'))
 
             if not project.row_count or not project.row:
-                raise problemdetails.Problem(
-                    status_code=404, title='Item not found')
+                raise errors.ItemNotFound()
 
             output = project.row
             output.update({
